# the logic is taken from llvmlite
import llvmpym.core as core
from llvmpym.core import IntType, IntPredicate

fn_type = core.FunctionType(IntType.GlobalInt32,
                            [IntType.GlobalInt32, IntType.GlobalInt32],
                            False)
m = core.Module("")
fn = core.Function(m, "foo", fn_type)
bb_entry = fn.append_basic_block("")

builder = core.Builder()
builder.position_at_end(bb_entry)

stackint = builder.alloca(IntType.GlobalInt32, name="")
assert isinstance(stackint.type, IntType) # remove lsp diagnostic
builder.store(core.ConstantInt(stackint.type, 123, True), stackint)
myint = builder.load2(stackint.type, stackint, "")

# Currently, every property goes through an function call through LLVM
# involving WRAP and UNWRAP multiple many times through both python - LLVM-C
# layer and LLVM-C - LLVM-C++ layer. There may be some performance loss.
# In the future, we may add cache to object properties.
add_inst = builder.add(fn.params[0], fn.params[1], "")
mul_inst = builder.mul(add_inst, core.ConstantInt(IntType.GlobalInt32, 123, True), "")
pred = builder.icmp(IntPredicate.SLE, add_inst, mul_inst, "")
builder.ret(mul_inst)

bb_block = fn.append_basic_block("")
builder.position_at_end(bb_block)

bb_exit = fn.append_basic_block("")

pred = builder.trunc(add_inst, IntType.GlobalInt1, "")
builder.cond_br(pred, bb_block, bb_exit)

builder.position_at_end(bb_exit)
builder.ret(myint)

print(m)




# Note use `pip install .` to install this package
# In `pip install --no-build-isolation -ve .` mode it won't work
from llvmpym.core import *

class TestContants:
    pass

class TestEnum:
    def test_enum_member_numbers(self):
        assert len(Opcode) == 67
        assert len(TypeKind) == 21
        assert len(Linkage) == 17
        assert len(Visibility) == 3
        assert len(UnnamedAddr) == 3
        assert len(DLLStorageClass) == 3
        assert len(CallConv) == 41
        assert len(ValueKind) == 27
        assert len(IntPredicate) == 10
        assert len(RealPredicate) == 16
        assert len(LandingPadClauseTy) == 2
        assert len(ThreadLocalMode) == 5
        assert len(AtomicOrdering) == 7
        assert len(AtomicRMWBinOp) == 15
        assert len(DiagnosticSeverity) == 4
        assert len(InlineAsmDialect) == 2
        assert len(ModuleFlagBehavior) == 6
        assert len(AttributeIndex) == 2
        assert len(TailCallKind) == 4
        assert len(FastMathFlags) == 9

        
    def test_value(self):
        assert IntPredicate.EQ.value == 32

    def test_docstring(self):
        assert TypeKind.Void.__doc__ == "type with no size"

        

class TestModule:
    def test_module(self):
        pass
    

